@@ -23,25 +23,29 @@
 #include "drkonqi.h"
 #include "crashedapplication.h"
 #include "backtracegenerator.h"
-#include "backtraceparser.h"
+#include "parser/backtraceparser.h"
 #include "drkonqi_globals.h"
 #include "debuggermanager.h"
+#include "gdbhighlighter.h"
 
 #include <QtGui/QLabel>
 #include <QtGui/QHBoxLayout>
 #include <QtGui/QVBoxLayout>
+#include <QtGui/QScrollBar>
 
 #include <KIcon>
 #include <KMessageBox>
 #include <KLocale>
 #include <KToolInvocation>
+#include <KGlobalSettings>
 
 const char *extraDetailsLabelMargin = " margin: 5px; ";
 
 BacktraceWidget::BacktraceWidget(BacktraceGenerator *generator, QWidget *parent,
     bool showToggleBacktrace) :
         QWidget(parent),
-        m_btGenerator(generator)
+        m_btGenerator(generator),
+        m_highlighter(0)
 {
     ui.setupUi(this);
 
@@ -116,10 +120,16 @@ BacktraceWidget::BacktraceWidget(BacktraceGenerator *generator, QWidget *parent,
                 SLOT(toggleBacktrace(bool)));
         toggleBacktrace(false);
     }
+
+    ui.m_backtraceEdit->setFont( KGlobalSettings::fixedFont() );
 }
 
 void BacktraceWidget::setAsLoading()
 {
+    //remove the syntax highlighter
+    delete m_highlighter;
+    m_highlighter = 0;
+
     //Set the widget as loading and disable all the action buttons
     ui.m_backtraceEdit->setText(i18nc("@info:status", "Loading..."));
     ui.m_backtraceEdit->setEnabled(false);
@@ -198,6 +208,19 @@ void BacktraceWidget::loadData()
         ui.m_backtraceEdit->setEnabled(true);
         ui.m_backtraceEdit->setPlainText(m_btGenerator->backtrace());
 
+        // scroll to crash
+        QTextCursor crashCursor = ui.m_backtraceEdit->document()->find("[KCrash Handler]");
+        if (!crashCursor.isNull()) {
+            crashCursor.movePosition(QTextCursor::Up, QTextCursor::MoveAnchor);
+            ui.m_backtraceEdit->verticalScrollBar()->setValue(ui.m_backtraceEdit->cursorRect(crashCursor).top());
+        }
+
+        // highlight if possible
+        if (m_btGenerator->debugger().codeName() == "gdb") {
+            m_highlighter = new GdbHighlighter(ui.m_backtraceEdit->document(),
+                                               m_btGenerator->parser()->parsedBacktraceLines());
+        }
+
         BacktraceParser * btParser = m_btGenerator->parser();
         m_backtraceRatingWidget->setUsefulness(btParser->backtraceUsefulness());
 
