@@ -29,6 +29,7 @@
 #include <QDir>
 #include <QFile>
 #include <QTextStream>
+#include <QTextDocument>
 #include <QMap>
 #include <QRegExp>
 #include <QTextCodec>
@@ -87,6 +88,7 @@ bool parseUrl(const QString& _url, QString &title, QString &section)
     section.clear();
 
     QString url = _url;
+    url = url.trimmed();
     if (url.isEmpty() || url.at(0) == '/') {
         if (url.isEmpty() || KStandardDirs::exists(url)) {
             // man:/usr/share/man/man1/ls.1.gz is a valid file
@@ -447,19 +449,6 @@ extern void output_real(const char *insert)
 }
 #endif
 
-static QString text2html(const QString& txt)
-{
-    QString reply = txt;
-
-    reply = reply.replace('&', "&amp;");
-    reply = reply.replace('<', "&lt;");
-    reply = reply.replace('>', "&gt;");
-    reply = reply.replace('"', "&dquot;");
-    reply = reply.replace('\'', "&quot;");
-    return reply;
-}
-
-
 void MANProtocol::get(const KUrl& url )
 {
     kDebug(7107) << "GET " << url.url();
@@ -495,7 +484,7 @@ void MANProtocol::get(const KUrl& url )
            "Check that you have typed the name using the correct upper and lower case characters.<br />"
            "If everything looks correct, then you may need to improve the search path "
            "for man pages; either using the environment variable MANPATH or using a matching file "
-           "in the /etc directory.", text2html(title)));
+           "in the /etc directory.", Qt::escape(title)));
        pageFound=false;
     }
     else if (foundPages.count()>1)
@@ -570,19 +559,24 @@ char *MANProtocol::readManPage(const char *_filename)
     }
     else
     {
-        if (QDir::isRelativePath(filename)) {
+        if (QDir::isRelativePath(filename))
+        {
             kDebug(7107) << "relative " << filename;
             filename = QDir::cleanPath(lastdir + '/' + filename).toUtf8();
-            if (!KStandardDirs::exists(filename)) { // exists perhaps with suffix
-                lastdir = filename.left(filename.lastIndexOf('/'));
-                QDir mandir(lastdir);
-                mandir.setNameFilters(QStringList() << (filename.mid(filename.lastIndexOf('/') + 1) + ".*"));
-                filename = lastdir + '/' + QFile::encodeName(mandir.entryList().first());
-            }
             kDebug(7107) << "resolved to " << filename;
         }
+
         lastdir = filename.left(filename.lastIndexOf('/'));
 
+        if ( !QFile::exists(QFile::decodeName(filename)) )  // if given file does not exist, find with suffix
+        {
+            kDebug(7107) << "not existing " << filename;
+            QDir mandir(lastdir);
+            mandir.setNameFilters(QStringList() << (filename.mid(filename.lastIndexOf('/') + 1) + ".*"));
+            filename = lastdir + '/' + QFile::encodeName(mandir.entryList().first());
+            kDebug(7107) << "resolved to " << filename;
+        }
+
         QIODevice *fd= KFilterDev::deviceForFile(filename);
 
         if ( !fd || !fd->open(QIODevice::ReadOnly))
@@ -1228,9 +1222,20 @@ void MANProtocol::showIndex(const QString& section)
     infoMessage(i18n("Generating Index"));
 
     // search for the man pages
-	QStringList pages = findPages( section, QString() );
+    QStringList pages = findPages( section, QString() );
 
-	QMap<QString, QString> indexmap = buildIndexMap(section);
+    if ( pages.count() == 0 )  // not a single page found
+    {
+      // print footer
+      os << "</div></body></html>" << endl;
+
+      infoMessage(QString());
+      data(array);
+      finished();
+      return;
+    }
+
+    QMap<QString, QString> indexmap = buildIndexMap(section);
 
     // print out the list
     os << "<table>" << endl;
@@ -1485,7 +1490,6 @@ void MANProtocol::showIndex(const QString& section)
     os << "</body></html>" << endl;
 
     infoMessage(QString());
-    mimeType("text/html");
     data(array);
     finished();
 }
