@@ -77,6 +77,9 @@ static const char* const SimpleServiceMimetype[] =
     "presence",
     "lobby",
     "giver",
+    "sip",
+    "h323",
+    "skype",
 
     "ipp",
     "printer",
@@ -92,10 +95,13 @@ static const char* const SimpleServiceMimetype[] =
     "pulse-server",
     "pulse-source",
     "pulse-sink",
+    "udisks-ssh",
     "libvirt",
     "airmouse",
 
     "postgresql",
+    "couchdb_location",
+
     "realplayfavs",
     "acrobat-server",
     "adobe-vc",
@@ -105,6 +111,8 @@ static const char* const SimpleServiceMimetype[] =
     "pgpkey-hkp",
     "pgpkey-https",
 
+    "maemo-inf",
+
     "airport",
     "daap",
     "dacp",
