@@ -50,29 +50,23 @@ K_EXPORT_PLUGIN(KPasswdServerFactory("kpasswdserver"))
 #define AUTHINFO_EXTRAFIELD_ANONYMOUS "anonymous"
 #define AUTHINFO_EXTRAFIELD_BYPASS_CACHE_AND_KWALLET "bypass-cache-and-kwallet"
 
-qlonglong getRequestId()
+static int debugArea() { static int s_area = KDebug::registerArea("KPasswdServer"); return s_area; }
+
+static qlonglong getRequestId()
 {
     static qlonglong nextRequestId = 0;
     return nextRequestId++;
 }
 
-int
-KPasswdServer::AuthInfoContainerList::compareItems(Q3PtrCollection::Item n1, Q3PtrCollection::Item n2)
+bool
+KPasswdServer::AuthInfoContainer::Sorter::operator ()(AuthInfoContainer* n1, AuthInfoContainer* n2) const
 {
    if (!n1 || !n2)
       return 0;
 
-   AuthInfoContainer *i1 = (AuthInfoContainer *) n1;
-   AuthInfoContainer *i2 = (AuthInfoContainer *) n2;
-
-   int l1 = i1->directory.length();
-   int l2 = i2->directory.length();
-
-   if (l1 > l2)
-      return -1;
-   if (l1 < l2)
-      return 1;
-   return 0;
+   const int l1 = n1->directory.length();
+   const int l2 = n2->directory.length();
+   return l1 < l2;
 }
 
 
@@ -95,7 +89,7 @@ KPasswdServer::KPasswdServer(QObject* parent, const QList<QVariant>&)
             SIGNAL(queryAuthInfoAsyncResult(qlonglong, qlonglong, const KIO::AuthInfo &)),
             adaptor,
             SIGNAL(queryAuthInfoAsyncResult(qlonglong, qlonglong, const KIO::AuthInfo &)));
-    
+
     connect(this, SIGNAL(windowUnregistered(qlonglong)),
             this, SLOT(removeAuthForWindowId(qlonglong)));
 }
@@ -137,7 +131,7 @@ static bool storeInWallet( KWallet::Wallet* wallet, const QString& key, const KI
     int entryNumber = 1;
     Map map;
     QString walletKey = makeWalletKey( key, info.realmValue );
-    kDebug(130) << "walletKey =" << walletKey << "  reading existing map";
+    kDebug(debugArea()) << "walletKey =" << walletKey << "  reading existing map";
     if ( wallet->readMap( walletKey, map ) == 0 ) {
         Map::ConstIterator end = map.constEnd();
         Map::ConstIterator it = map.constFind( "login" );
@@ -151,7 +145,7 @@ static bool storeInWallet( KWallet::Wallet* wallet, const QString& key, const KI
     }
     const QString loginKey = makeMapKey( "login", entryNumber );
     const QString passwordKey = makeMapKey( "password", entryNumber );
-    kDebug(130) << "writing to " << loginKey << "," << passwordKey;
+    kDebug(debugArea()) << "writing to " << loginKey << "," << passwordKey;
     // note the overwrite=true by default
     map.insert( loginKey, info.username );
     map.insert( passwordKey, info.password );
@@ -162,7 +156,7 @@ static bool storeInWallet( KWallet::Wallet* wallet, const QString& key, const KI
 
 static bool readFromWallet( KWallet::Wallet* wallet, const QString& key, const QString& realm, QString& username, QString& password, bool userReadOnly, QMap<QString,QString>& knownLogins )
 {
-    //kDebug(130) << "key =" << key << " username =" << username << " password =" /*<< password*/ << " userReadOnly =" << userReadOnly << " realm =" << realm;
+    //kDebug(debugArea()) << "key =" << key << " username =" << username << " password =" /*<< password*/ << " userReadOnly =" << userReadOnly << " realm =" << realm;
     if ( wallet->hasFolder( KWallet::Wallet::PasswordFolder() ) )
     {
         wallet->setFolder( KWallet::Wallet::PasswordFolder() );
@@ -175,7 +169,7 @@ static bool readFromWallet( KWallet::Wallet* wallet, const QString& key, const Q
             Map::ConstIterator end = map.constEnd();
             Map::ConstIterator it = map.constFind( "login" );
             while ( it != end ) {
-                //kDebug(130) << "found " << it.key() << "=" << it.value();
+                //kDebug(debugArea()) << "found " << it.key() << "=" << it.value();
                 Map::ConstIterator pwdIter = map.constFind( makeMapKey( "password", entryNumber ) );
                 if ( pwdIter != end ) {
                     if ( it.value() == username )
@@ -185,13 +179,13 @@ static bool readFromWallet( KWallet::Wallet* wallet, const QString& key, const Q
 
                 it = map.constFind( QString( "login-" ) + QString::number( ++entryNumber ) );
             }
-            //kDebug(130) << knownLogins.count() << " known logins";
+            //kDebug(debugArea()) << knownLogins.count() << " known logins";
 
             if ( !userReadOnly && !knownLogins.isEmpty() && username.isEmpty() ) {
                 // Pick one, any one...
                 username = knownLogins.begin().key();
                 password = knownLogins.begin().value();
-                //kDebug(130) << "picked the first one:" << username;
+                //kDebug(debugArea()) << "picked the first one:" << username;
             }
 
             return true;
@@ -228,9 +222,10 @@ KPasswdServer::checkAuthInfo(const QByteArray &data, qlonglong windowId, qlonglo
     KIO::AuthInfo info;
     QDataStream stream(data);
     stream >> info;
-    kDebug(130) << "User =" << info.username << ", WindowId =" << windowId << endl;
-    if( usertime != 0 )
-        kapp->updateUserTimestamp( usertime );
+    kDebug(debugArea()) << "User =" << info.username << ", WindowId =" << windowId << endl;
+    if (usertime != 0) {
+        kapp->updateUserTimestamp(usertime);
+    }
 
     // if the check depends on a pending query, delay it
     // until that query is finished.
@@ -239,7 +234,9 @@ KPasswdServer::checkAuthInfo(const QByteArray &data, qlonglong windowId, qlonglo
         setDelayedReply(true);
         Request *pendingCheck = new Request;
         pendingCheck->isAsync = false;
-        pendingCheck->transaction = message();
+        if (calledFromDBus()) {
+            pendingCheck->transaction = message();
+        }
         pendingCheck->key = key;
         pendingCheck->info = info;
         m_authWait.append(pendingCheck);
@@ -271,7 +268,7 @@ KPasswdServer::checkAuthInfo(const QByteArray &data, qlonglong windowId, qlonglo
         updateAuthExpire(key, result, windowId, false);
         info = copyAuthInfo(result);
     }
-    
+
     QByteArray data2;
     QDataStream stream2(&data2, QIODevice::WriteOnly);
     stream2 << info;
@@ -281,15 +278,17 @@ KPasswdServer::checkAuthInfo(const QByteArray &data, qlonglong windowId, qlonglo
 qlonglong KPasswdServer::checkAuthInfoAsync(KIO::AuthInfo info, qlonglong windowId,
                                             qlonglong usertime)
 {
-    kDebug(130) << "User =" << info.username << ", WindowId =" << windowId << endl;
     if (usertime != 0) {
         kapp->updateUserTimestamp(usertime);
     }
 
     // send the request id back to the client
     qlonglong requestId = getRequestId();
-    QDBusMessage reply(message().createReply(requestId));
-    QDBusConnection::sessionBus().send(reply);
+    kDebug(debugArea()) << "User =" << info.username << ", WindowId =" << windowId;
+    if (calledFromDBus()) {
+        QDBusMessage reply(message().createReply(requestId));
+        QDBusConnection::sessionBus().send(reply);
+    }
 
     // if the check depends on a pending query, delay it
     // until that query is finished.
@@ -341,12 +340,13 @@ KPasswdServer::queryAuthInfo(const QByteArray &data, const QString &errorMsg,
     KIO::AuthInfo info;
     QDataStream stream(data);
     stream >> info;
-    kDebug(130) << "User =" << info.username << ", Message= " << info.prompt
+    kDebug(debugArea()) << "User =" << info.username << ", Message= " << info.prompt
                 << ", WindowId =" << windowId << endl;
     if ( !info.password.isEmpty() ) // should we really allow the caller to pre-fill the password?
-        kDebug(130) << "password was set by caller";
-    if( usertime != 0 )
-        kapp->updateUserTimestamp( usertime );
+        kDebug(debugArea()) << "password was set by caller";
+    if (usertime != 0) {
+        kapp->updateUserTimestamp(usertime);
+    }
 
     QString key = createCacheKey(info);
     Request *request = new Request;
@@ -379,12 +379,10 @@ qlonglong
 KPasswdServer::queryAuthInfoAsync(const KIO::AuthInfo &info, const QString &errorMsg,
                                   qlonglong windowId, qlonglong seqNr, qlonglong usertime)
 {
-/*    KIO::AuthInfo info;
-    data >> info; */
-    kDebug(130) << "User =" << info.username << ", Message= " << info.prompt
+    kDebug(debugArea()) << "User =" << info.username << ", Message= " << info.prompt
                 << ", WindowId =" << windowId << endl;
     if (!info.password.isEmpty()) {
-        kDebug(130) << "password was set by caller";
+        kDebug(debugArea()) << "password was set by caller";
     }
     if (usertime != 0) {
         kapp->updateUserTimestamp(usertime);
@@ -417,22 +415,7 @@ KPasswdServer::queryAuthInfoAsync(const KIO::AuthInfo &info, const QString &erro
 void
 KPasswdServer::addAuthInfo(const KIO::AuthInfo &info, qlonglong windowId)
 {
-    kDebug(130) << "User =" << info.username << ", RealmValue= " << info.realmValue
-                << ", WindowId = " << windowId << endl;
-    QString key = createCacheKey(info);
-
-    m_seqNr++;
-
-    addAuthInfoItem(key, info, windowId, m_seqNr, false);
-}    
-
-void
-KPasswdServer::addAuthInfo(const QByteArray &data, qlonglong windowId)
-{
-    KIO::AuthInfo info;
-    QDataStream stream(data);
-    stream >> info;
-    kDebug(130) << "User =" << info.username << ", RealmValue= " << info.realmValue
+    kDebug(debugArea()) << "User =" << info.username << ", RealmValue= " << info.realmValue
                 << ", WindowId = " << windowId << endl;
     QString key = createCacheKey(info);
 
@@ -441,13 +424,24 @@ KPasswdServer::addAuthInfo(const QByteArray &data, qlonglong windowId)
     addAuthInfoItem(key, info, windowId, m_seqNr, false);
 }
 
+void
+KPasswdServer::addAuthInfo(const QByteArray &data, qlonglong windowId)
+{
+    KIO::AuthInfo info;
+    QDataStream stream(data);
+    stream >> info;
+    kDebug(debugArea()) << "User =" << info.username << ", RealmValue= " << info.realmValue
+                << ", WindowId = " << windowId << endl;
+    addAuthInfo(info, windowId);
+}
+
 void
 KPasswdServer::removeAuthInfo(const QString& host, const QString& protocol, const QString& user)
 {
-    kDebug(130) << "Wanted" << protocol << host << user;
+    kDebug(debugArea()) << "Wanted" << protocol << host << user;
 
     QHashIterator< QString, AuthInfoContainerList* > dictIterator(m_authDict);
-    while (dictIterator.hasNext()) 
+    while (dictIterator.hasNext())
     {
         dictIterator.next();
 
@@ -455,22 +449,17 @@ KPasswdServer::removeAuthInfo(const QString& host, const QString& protocol, cons
         if (!authList)
             continue;
 
-        for(AuthInfoContainer *current = authList->first(); current; )
+        Q_FOREACH(AuthInfoContainer *current, *authList)
         {
-            kDebug(130) << "Evaluating: " << current->info.url.protocol() 
-                     << current->info.url.host() 
+            kDebug(debugArea()) << "Evaluating: " << current->info.url.protocol()
+                     << current->info.url.host()
                      << current->info.username;
             if (current->info.url.protocol() == protocol &&
                current->info.url.host() == host &&
                (current->info.username == user || user.isEmpty()))
             {
-                kDebug(130) << "Removing this entry";
+                kDebug(debugArea()) << "Removing this entry";
                 removeAuthInfoItem(dictIterator.key(), current->info);
-                current = authList->current();
-            }
-            else
-            {
-                current = authList->next();
             }
         }
     }
@@ -494,24 +483,21 @@ KPasswdServer::processRequest()
 {
     // guard so processRequest is only ever run once.
     static bool processing = false;
-    if (processing) {
+    if (processing || m_authPending.isEmpty()) {
         return;
     }
     processing = true;
-    
-    Request *request = m_authPending.takeFirst();
-    if (!request)
-       return;
 
+    Request *request = m_authPending.takeFirst();
     KIO::AuthInfo &info = request->info;
     bool bypassCacheAndKWallet = info.getExtraField(AUTHINFO_EXTRAFIELD_BYPASS_CACHE_AND_KWALLET).toBool() == true;
 
-    kDebug(130) << "User =" << info.username << ", Message =" << info.prompt << endl;
+    kDebug(debugArea()) << "User =" << info.username << ", Message =" << info.prompt;
     const AuthInfoContainer *result = findAuthInfoItem(request->key, request->info);
 
     if (!bypassCacheAndKWallet && result && (request->seqNr < result->seqNr))
     {
-        kDebug(130) << "auto retry!";
+        kDebug(debugArea()) << "auto retry!";
         if (result->isCanceled)
         {
            info.setModified(false);
@@ -526,8 +512,7 @@ KPasswdServer::processRequest()
     {
         m_seqNr++;
         bool askPw = request->prompt;
-        if (result && !info.username.isEmpty() &&
-            !request->errorMsg.isEmpty())
+        if (result && !request->errorMsg.isEmpty())
         {
            QString prompt = request->errorMsg + "  ";
            prompt += i18n("Do you want to retry?");
@@ -600,7 +585,7 @@ KPasswdServer::processRequest()
             if (info.getExtraField(AUTHINFO_EXTRAFIELD_DOMAIN).isValid ())
                 dlg.setDomain(info.getExtraField(AUTHINFO_EXTRAFIELD_DOMAIN).toString());
 
-            if (info.getExtraField(AUTHINFO_EXTRAFIELD_ANONYMOUS).isValid ())
+            if (info.getExtraField(AUTHINFO_EXTRAFIELD_ANONYMOUS).isValid () && password.isEmpty() && username.isEmpty())
                 dlg.setAnonymousMode(info.getExtraField(AUTHINFO_EXTRAFIELD_ANONYMOUS).toBool());
 
 #ifndef Q_WS_WIN
@@ -609,7 +594,9 @@ KPasswdServer::processRequest()
             KWindowSystem::setMainWindow(&dlg, (HWND)(long)request->windowId);
 #endif
 
+            kDebug() << "Calling exec on" << &dlg;
             dlgResult = dlg.exec();
+            kDebug() << "exec returned with" << dlgResult;
 
             if (dlgResult == QDialog::Accepted)
             {
@@ -704,7 +691,7 @@ KPasswdServer::processRequest()
 
     // reallow processing
     processing = false;
-    
+
     if (m_authPending.count())
        QTimer::singleShot(0, this, SLOT(processRequest()));
 
@@ -714,7 +701,7 @@ QString KPasswdServer::createCacheKey( const KIO::AuthInfo &info )
 {
     if( !info.url.isValid() ) {
         // Note that a null key will break findAuthInfoItem later on...
-        kWarning(130) << "createCacheKey: invalid URL " << info.url ;
+        kWarning(debugArea()) << "createCacheKey: invalid URL " << info.url ;
         return QString();
     }
 
@@ -754,14 +741,13 @@ KPasswdServer::findAuthInfoItem(const QString &key, const KIO::AuthInfo &info)
       return 0;
 
    QString path2 = info.url.directory(KUrl::AppendTrailingSlash|KUrl::ObeyTrailingSlash);
-   for(AuthInfoContainer *current = authList->first();
-       current; )
+   Q_FOREACH(AuthInfoContainer *current, *authList)
    {
        if ((current->expire == AuthInfoContainer::expTime) &&
           (difftime(time(0), current->expireTime) > 0))
        {
-          authList->remove();
-          current = authList->current();
+          delete current;
+          authList->removeOne(current);
           continue;
        }
 
@@ -778,8 +764,6 @@ KPasswdServer::findAuthInfoItem(const QString &key, const KIO::AuthInfo &info)
               (info.username.isEmpty() || info.username == current->info.username))
              return current; // TODO: Update directory info,
        }
-
-       current = authList->next();
    }
    return 0;
 }
@@ -791,17 +775,12 @@ KPasswdServer::removeAuthInfoItem(const QString &key, const KIO::AuthInfo &info)
    if (!authList)
       return;
 
-   for(AuthInfoContainer *current = authList->first();
-       current; )
+   Q_FOREACH(AuthInfoContainer *current, *authList)
    {
        if (current->info.realmValue == info.realmValue)
        {
-          authList->remove();
-          current = authList->current();
-       }
-       else
-       {
-          current = authList->next();
+          authList->removeOne(current);
+          delete current;
        }
    }
    if (authList->isEmpty())
@@ -820,12 +799,12 @@ KPasswdServer::addAuthInfoItem(const QString &key, const KIO::AuthInfo &info, ql
       authList = new AuthInfoContainerList;
       m_authDict.insert(key, authList);
    }
-   AuthInfoContainer *current = authList->first();
-   for(; current; current = authList->next())
+   AuthInfoContainer *current = 0;
+   Q_FOREACH(current, *authList)
    {
        if (current->info.realmValue == info.realmValue)
        {
-          authList->take();
+          authList->removeOne(current);
           break;
        }
    }
@@ -834,11 +813,11 @@ KPasswdServer::addAuthInfoItem(const QString &key, const KIO::AuthInfo &info, ql
    {
       current = new AuthInfoContainer;
       current->expire = AuthInfoContainer::expTime;
-      kDebug(130) << "Creating AuthInfoContainer";
+      kDebug(debugArea()) << "Creating AuthInfoContainer";
    }
    else
    {
-      kDebug(130) << "Updating AuthInfoContainer";
+      kDebug(debugArea()) << "Updating AuthInfoContainer";
    }
 
    current->info = info;
@@ -849,7 +828,8 @@ KPasswdServer::addAuthInfoItem(const QString &key, const KIO::AuthInfo &info, ql
    updateAuthExpire(key, current, windowId, info.keepPassword && !canceled);
 
    // Insert into list, keep the list sorted "longest path" first.
-   authList->inSort(current);
+   authList->append(current);
+   qSort(authList->begin(), authList->end(), AuthInfoContainer::Sorter());
 }
 
 void
@@ -874,42 +854,32 @@ KPasswdServer::updateAuthExpire(const QString &key, const AuthInfoContainer *aut
    // Update mWindowIdList
    if (windowId)
    {
-      QStringList *keysChanged = mWindowIdList.value(windowId);
-      if (!keysChanged)
-      {
-         keysChanged = new QStringList;
-         mWindowIdList.insert(windowId, keysChanged);
-      }
-      if (!keysChanged->contains(key))
-         keysChanged->append(key);
+      QStringList& keysChanged = mWindowIdList[windowId]; // find or insert
+      if (!keysChanged.contains(key))
+         keysChanged.append(key);
    }
 }
 
 void
 KPasswdServer::removeAuthForWindowId(qlonglong windowId)
 {
-   QStringList *keysChanged = mWindowIdList.value(windowId);
-   if (!keysChanged) return;
-
-   foreach (const QString &key, *keysChanged)
+   const QStringList keysChanged = mWindowIdList.value(windowId);
+   foreach (const QString &key, keysChanged)
    {
       AuthInfoContainerList *authList = m_authDict.value(key);
       if (!authList)
          continue;
 
-      AuthInfoContainer *current = authList->first();
-      for(; current; )
+      Q_FOREACH(AuthInfoContainer *current, *authList)
       {
         if (current->expire == AuthInfoContainer::expWindowClose)
         {
            if (current->windowList.removeAll(windowId) && current->windowList.isEmpty())
            {
-              authList->remove();
-              current = authList->current();
-              continue;
+              authList->removeOne(current);
+              delete current;
            }
         }
-        current = authList->next();
       }
    }
 }
