@@ -1,5 +1,6 @@
 /* This file is part of the KDE Project
    Copyright (c) 2008-2010 Sebastian Trueg <trueg@kde.org>
+   Copyright (c) 2010 Vishesh Handa <handa.vish@gmail.com>
 
    Parts of this file are based on code from Strigi
    Copyright (C) 2006-2007 Jos van den Oever <jos@vandenoever.info>
@@ -21,8 +22,8 @@
 
 #include "indexscheduler.h"
 #include "strigiserviceconfig.h"
-#include "nfo.h"
-#include "nie.h"
+#include "nepomukindexer.h"
+#include "util.h"
 
 #include <QtCore/QMutexLocker>
 #include <QtCore/QList>
@@ -32,7 +33,6 @@
 #include <QtCore/QDateTime>
 #include <QtCore/QByteArray>
 #include <QtCore/QUrl>
-#include <QtCore/QTimer>
 
 #include <KDebug>
 #include <KTemporaryFile>
@@ -41,29 +41,29 @@
 #include <Nepomuk/Resource>
 #include <Nepomuk/ResourceManager>
 #include <Nepomuk/Variant>
+#include <Nepomuk/Query/Query>
+#include <Nepomuk/Query/ComparisonTerm>
+#include <Nepomuk/Query/ResourceTerm>
 
 #include <Soprano/Model>
 #include <Soprano/QueryResultIterator>
 #include <Soprano/NodeIterator>
 #include <Soprano/Node>
+
+#include <Soprano/Vocabulary/RDF>
 #include <Soprano/Vocabulary/Xesam>
+#include <Nepomuk/Vocabulary/NFO>
+#include <Nepomuk/Vocabulary/NIE>
 
 #include <map>
 #include <vector>
 
-#include <strigi/strigiconfig.h>
-#include <strigi/indexwriter.h>
-#include <strigi/indexmanager.h>
-#include <strigi/indexreader.h>
-#include <strigi/analysisresult.h>
-#include <strigi/fileinputstream.h>
-#include <strigi/analyzerconfiguration.h>
-
 
 namespace {
     const int s_reducedSpeedDelay = 500; // ms
     const int s_snailPaceDelay = 3000;   // ms
 
+
     bool isResourcePresent( const QString & dir ) {
         QString query = QString::fromLatin1(" ask { ?r %1 %2. } ")
                         .arg( Soprano::Node::resourceToN3( Nepomuk::Vocabulary::NIE::url() ),
@@ -110,52 +110,61 @@ namespace {
 }
 
 
-class Nepomuk::IndexScheduler::StoppableConfiguration : public Strigi::AnalyzerConfiguration
+void Nepomuk::IndexScheduler::UpdateDirQueue::enqueueDir( const QString& dir, UpdateDirFlags flags )
 {
-public:
-    StoppableConfiguration()
-        : m_stop(false) {
-#if defined(STRIGI_IS_VERSION)
-#if STRIGI_IS_VERSION( 0, 6, 1 )
-        setIndexArchiveContents( false );
-#endif
-#endif
+    if( contains( qMakePair( dir, flags ) ) )
+        return;
+
+    if( !(flags & AutoUpdateFolder) ) {
+        int i = 0;
+        while( i < count() && !(at(i).second & AutoUpdateFolder) )
+            ++i;
+        insert( i, qMakePair( dir, flags ) );
     }
-
-    bool indexMore() const {
-        return !m_stop;
+    else {
+        enqueue( qMakePair( dir, flags ) );
     }
+}
 
-    bool addMoreText() const {
-        return !m_stop;
+
+void Nepomuk::IndexScheduler::UpdateDirQueue::prependDir( const QString& dir, UpdateDirFlags flags )
+{
+    if( contains( qMakePair( dir, flags ) ) )
+        return;
+
+    if( flags & AutoUpdateFolder ) {
+        int i = 0;
+        while( i < count() && !(at(i).second & AutoUpdateFolder) )
+            ++i;
+        insert( i, qMakePair( dir, flags ) );
     }
-
-    void setStop( bool s ) {
-        m_stop = s;
+    else {
+        prepend( qMakePair( dir, flags ) );
     }
-
-private:
-    bool m_stop;
-};
+}
 
 
-Nepomuk::IndexScheduler::IndexScheduler( Strigi::IndexManager* manager, QObject* parent )
+void Nepomuk::IndexScheduler::UpdateDirQueue::clearByFlags( UpdateDirFlags mask )
+{
+    QQueue<QPair<QString, UpdateDirFlags> >::iterator it = begin();
+    while ( it != end() ) {
+        if ( it->second & mask )
+            it = erase( it );
+        else
+            ++it;
+    }
+}
+
+
+
+Nepomuk::IndexScheduler::IndexScheduler( QObject* parent )
     : QThread( parent ),
       m_suspended( false ),
       m_stopped( false ),
       m_indexing( false ),
-      m_indexManager( manager ),
       m_speed( FullSpeed )
 {
-    m_analyzerConfig = new StoppableConfiguration;
-
-    // see updateDir(QString,bool) for details on the timer
-    m_dirsToUpdateWakeupTimer = new QTimer( this );
-    m_dirsToUpdateWakeupTimer->setSingleShot( true );
-    m_dirsToUpdateWakeupTimer->setInterval(1000);
-    connect( m_dirsToUpdateWakeupTimer, SIGNAL( timeout() ),
-             this, SLOT( slotDirsToUpdateWakeupTimeout() ) );
-
+    m_indexer = new Nepomuk::Indexer( this );
     connect( StrigiServiceConfig::self(), SIGNAL( configChanged() ),
              this, SLOT( slotConfigChanged() ) );
 }
@@ -163,7 +172,6 @@ Nepomuk::IndexScheduler::IndexScheduler( Strigi::IndexManager* manager, QObject*
 
 Nepomuk::IndexScheduler::~IndexScheduler()
 {
-    delete m_analyzerConfig;
 }
 
 
@@ -203,7 +211,7 @@ void Nepomuk::IndexScheduler::stop()
         QMutexLocker locker( &m_resumeStopMutex );
         m_stopped = true;
         m_suspended = false;
-        m_analyzerConfig->setStop( true );
+        m_indexer->stop();
         m_dirsToUpdateWc.wakeAll();
         m_resumeStopWc.wakeAll();
     }
@@ -262,6 +270,7 @@ void Nepomuk::IndexScheduler::setIndexingStarted( bool started )
 {
     if ( started != m_indexing ) {
         m_indexing = started;
+        emit indexingStateChanged( m_indexing );
         if ( m_indexing )
             emit indexingStarted();
         else
@@ -274,30 +283,46 @@ void Nepomuk::IndexScheduler::run()
 {
     // set lowest priority for this thread
     setPriority( QThread::IdlePriority );
-
+    m_indexer->start();
+    
     setIndexingStarted( true );
 
     // initialization
     queueAllFoldersForUpdate();
 
+#ifndef NDEBUG
+    QTime timer;
+    timer.start();
+#endif
+
     removeOldAndUnwantedEntries();
 
-    Strigi::StreamAnalyzer analyzer( *m_analyzerConfig );
-    analyzer.setIndexWriter( *m_indexManager->indexWriter() );
+#ifndef NDEBUG
+    kDebug() << "Removed old entries: " << timer.elapsed()/1000.0 << "secs";
+    timer.restart();
+#endif
 
-    while ( waitForContinue() ) {
+    while ( waitForContinue(true) ) {
         // wait for more dirs to analyze in case the initial
         // indexing is done
+        m_dirsToUpdateMutex.lock();
         if ( m_dirsToUpdate.isEmpty() ) {
             setIndexingStarted( false );
 
-            m_dirsToUpdateMutex.lock();
+#ifndef NDEBUG
+            kDebug() << "All folders updated: " << timer.elapsed()/1000.0 << "sec";
+#endif
+
             m_dirsToUpdateWc.wait( &m_dirsToUpdateMutex );
-            m_dirsToUpdateMutex.unlock();
+
+#ifndef NDEBUG
+            timer.restart();
+#endif
 
             if ( !m_stopped )
                 setIndexingStarted( true );
         }
+        m_dirsToUpdateMutex.unlock();
 
         // wait for resume or stop (or simply continue)
         if ( !waitForContinue() ) {
@@ -306,12 +331,11 @@ void Nepomuk::IndexScheduler::run()
 
         // get the next folder
         m_dirsToUpdateMutex.lock();
-        QPair<QString, UpdateDirFlags> dir = *m_dirsToUpdate.begin();
-        m_dirsToUpdate.erase( m_dirsToUpdate.begin() );
+        QPair<QString, UpdateDirFlags> dir = m_dirsToUpdate.dequeue();
         m_dirsToUpdateMutex.unlock();
 
         // update until stopped
-        if ( !updateDir( dir.first, &analyzer, dir.second | UpdateRecursive ) ) {
+        if ( !analyzeDir( dir.first, dir.second | UpdateRecursive ) ) {
             break;
         }
         m_currentFolder.clear();
@@ -322,15 +346,20 @@ void Nepomuk::IndexScheduler::run()
     // reset state
     m_suspended = false;
     m_stopped = false;
-    m_analyzerConfig->setStop( false );
     m_currentFolder.clear();
 }
 
 
-bool Nepomuk::IndexScheduler::updateDir( const QString& dir, Strigi::StreamAnalyzer* analyzer, UpdateDirFlags flags )
+bool Nepomuk::IndexScheduler::analyzeDir( const QString& dir_, UpdateDirFlags flags )
 {
 //    kDebug() << dir << analyzer << recursive;
 
+    // normalize the dir name, otherwise things might break below
+    QString dir( dir_ );
+    if( dir.endsWith(QLatin1String("/")) ) {
+        dir.truncate( dir.length()-1 );
+    }
+
     // inform interested clients
     emit indexingFolder( dir );
 
@@ -344,7 +373,7 @@ bool Nepomuk::IndexScheduler::updateDir( const QString& dir, Strigi::StreamAnaly
     Nepomuk::Resource dirRes( dirUrl );
     if ( !dirRes.exists() ||
          dirRes.property( Nepomuk::Vocabulary::NIE::lastModified() ).toDateTime() != dirInfo.lastModified() ) {
-        analyzeFile( dirInfo, analyzer );
+        m_indexer->indexFile( dirInfo );
     }
 
     // get a map of all indexed files from the dir including their stored mtime
@@ -352,7 +381,6 @@ bool Nepomuk::IndexScheduler::updateDir( const QString& dir, Strigi::StreamAnaly
     QHash<QString, QDateTime>::iterator filesInStoreEnd = filesInStore.end();
 
     QList<QFileInfo> filesToIndex;
-    QStringList subFolders;
     QStringList filesToDelete;
 
     // iterate over all files in the dir
@@ -363,7 +391,7 @@ bool Nepomuk::IndexScheduler::updateDir( const QString& dir, Strigi::StreamAnaly
         QString path = dirIt.next();
 
         // FIXME: we cannot use canonialFilePath here since that could lead into another folder. Thus, we probably
-        // need to use another approach then the getChildren one.
+        // need to use another approach than the getChildren one.
         QFileInfo fileInfo = dirIt.fileInfo();//.canonialFilePath();
 
         bool indexFile = Nepomuk::StrigiServiceConfig::self()->shouldFileBeIndexed( fileInfo.fileName() );
@@ -378,6 +406,8 @@ bool Nepomuk::IndexScheduler::updateDir( const QString& dir, Strigi::StreamAnaly
         bool fileChanged = !newFile && fileInfo.lastModified() != filesInStoreIt.value();
         if ( fileChanged )
             kDebug() << "CHANGED:" << path << fileInfo.lastModified() << filesInStoreIt.value();
+        else if( forceUpdate )
+            kDebug() << "UPDATE FORCED:" << path;
 
         if ( indexFile && ( newFile || fileChanged || forceUpdate ) )
             filesToIndex << fileInfo;
@@ -391,8 +421,15 @@ bool Nepomuk::IndexScheduler::updateDir( const QString& dir, Strigi::StreamAnaly
         if ( !newFile )
             filesInStore.erase( filesInStoreIt );
 
-        if ( indexFile && recursive && fileInfo.isDir() && !fileInfo.isSymLink() )
-            subFolders << path;
+        // prepend sub folders to the dir queue
+        if ( indexFile &&
+                recursive &&
+                fileInfo.isDir() &&
+                !fileInfo.isSymLink() &&
+                StrigiServiceConfig::self()->shouldFolderBeIndexed( path ) ) {
+            QMutexLocker lock( &m_dirsToUpdateMutex );
+            m_dirsToUpdate.prependDir( path, flags );
+        }
     }
 
     // all the files left in filesInStore are not in the current
@@ -410,53 +447,14 @@ bool Nepomuk::IndexScheduler::updateDir( const QString& dir, Strigi::StreamAnaly
             return false;
 
         m_currentUrl = file.filePath();
-        analyzeFile( file, analyzer );
+        m_indexer->indexFile( file );
         m_currentUrl = KUrl();
     }
 
-    // recurse into subdirs (we do this in a separate loop to always keep a proper state:
-    // compare m_currentFolder)
-    if ( recursive ) {
-        foreach( const QString& folder, subFolders ) {
-            if ( StrigiServiceConfig::self()->shouldFolderBeIndexed( folder ) &&
-                 !updateDir( folder, analyzer, flags ) )
-                return false;
-        }
-    }
-
     return true;
 }
 
 
-void Nepomuk::IndexScheduler::analyzeFile( const QFileInfo& file, Strigi::StreamAnalyzer* analyzer )
-{
-    //
-    // strigi asserts if the file path has a trailing slash
-    //
-    QString filePath = m_currentUrl.toLocalFile( KUrl::RemoveTrailingSlash );
-    QString dir = m_currentUrl.directory(KUrl::IgnoreTrailingSlash);
-
-    Strigi::AnalysisResult analysisresult( QFile::encodeName( filePath ).data(),
-                                           file.lastModified().toTime_t(),
-                                           *m_indexManager->indexWriter(),
-                                           *analyzer,
-                                           QFile::encodeName( dir ).data() );
-    if ( file.isFile() && !file.isSymLink() ) {
-#ifdef STRIGI_HAS_FILEINPUTSTREAM_OPEN
-        Strigi::InputStream* stream = Strigi::FileInputStream::open( QFile::encodeName( file.filePath() ) );
-        analysisresult.index( stream );
-        delete stream;
-#else
-        Strigi::FileInputStream stream( QFile::encodeName( file.filePath() ) );
-        analysisresult.index( &stream );
-#endif
-    }
-    else {
-        analysisresult.index(0);
-    }
-}
-
-
 bool Nepomuk::IndexScheduler::waitForContinue( bool disableDelay )
 {
     QMutexLocker locker( &m_resumeStopMutex );
@@ -476,16 +474,8 @@ bool Nepomuk::IndexScheduler::waitForContinue( bool disableDelay )
 void Nepomuk::IndexScheduler::updateDir( const QString& path, bool forceUpdate )
 {
     QMutexLocker lock( &m_dirsToUpdateMutex );
-    m_dirsToUpdate << qMakePair( path, UpdateDirFlags( forceUpdate ? ForceUpdate : NoUpdateFlags ) );
-
-    // sometimes the filewatch service will call this method many times in a row with the same
-    // folder (in case a whole folder is created or modified). In order not to run an update every
-    // time we slightly delay the update process
-    if ( !isSuspended() &&
-         isIndexing() &&
-         !m_dirsToUpdateWakeupTimer->isActive() ) {
-        m_dirsToUpdateWakeupTimer->start();
-    }
+    m_dirsToUpdate.prependDir( path, UpdateDirFlags( forceUpdate ? ForceUpdate : NoUpdateFlags ) );
+    m_dirsToUpdateWc.wakeAll();
 }
 
 
@@ -501,21 +491,16 @@ void Nepomuk::IndexScheduler::queueAllFoldersForUpdate( bool forceUpdate )
     QMutexLocker lock( &m_dirsToUpdateMutex );
 
     // remove previously added folders to not index stuff we are not supposed to
-    QSet<QPair<QString, UpdateDirFlags> >::iterator it = m_dirsToUpdate.begin();
-    while ( it != m_dirsToUpdate.end() ) {
-        if ( it->second & AutoUpdateFolder )
-            it = m_dirsToUpdate.erase( it );
-        else
-            ++it;
-    }
+    m_dirsToUpdate.clearByFlags( AutoUpdateFolder );
 
     UpdateDirFlags flags = UpdateRecursive|AutoUpdateFolder;
     if ( forceUpdate )
         flags |= ForceUpdate;
 
     // update everything again in case the folders changed
-    foreach( const QString& f, StrigiServiceConfig::self()->includeFolders() )
-        m_dirsToUpdate << qMakePair( f, flags );
+    foreach( const QString& f, StrigiServiceConfig::self()->includeFolders() ) {
+        m_dirsToUpdate.enqueueDir( f, flags );
+    }
 }
 
 
@@ -527,67 +512,17 @@ void Nepomuk::IndexScheduler::slotConfigChanged()
 }
 
 
-void Nepomuk::IndexScheduler::slotDirsToUpdateWakeupTimeout()
-{
-    m_dirsToUpdateWc.wakeAll();
-}
-
-
-namespace {
-    class QDataStreamStrigiBufferedStream : public Strigi::BufferedStream<char>
-    {
-    public:
-        QDataStreamStrigiBufferedStream( QDataStream& stream )
-            : m_stream( stream ) {
-        }
-
-        int32_t fillBuffer( char* start, int32_t space ) {
-            int r = m_stream.readRawData( start, space );
-            if ( r == 0 ) {
-                // Strigi's API is so weird!
-                return -1;
-            }
-            else if ( r < 0 ) {
-                // Again: weird API. m_status is a protected member of StreamBaseBase (yes, 2x Base)
-                m_status = Strigi::Error;
-                return -1;
-            }
-            else {
-                return r;
-            }
-        }
-
-    private:
-        QDataStream& m_stream;
-    };
-}
-
-
 void Nepomuk::IndexScheduler::analyzeResource( const QUrl& uri, const QDateTime& modificationTime, QDataStream& data )
 {
-    Resource dirRes( uri );
-    if ( !dirRes.exists() ||
-         dirRes.property( Nepomuk::Vocabulary::NIE::lastModified() ).toDateTime() != modificationTime ) {
-        Strigi::StreamAnalyzer analyzer( *m_analyzerConfig );
-        analyzer.setIndexWriter( *m_indexManager->indexWriter() );
-        Strigi::AnalysisResult analysisresult( uri.toEncoded().data(),
-                                               modificationTime.toTime_t(),
-                                               *m_indexManager->indexWriter(),
-                                               analyzer );
-        QDataStreamStrigiBufferedStream stream( data );
-        analysisresult.index( &stream );
-    }
-    else {
-        kDebug() << uri << "up to date";
-    }
+    Indexer indexer;
+    indexer.indexResource( uri, modificationTime, data );
 }
 
 
 void Nepomuk::IndexScheduler::analyzeFile( const QString& path )
 {
-    Strigi::StreamAnalyzer analyzer( *m_analyzerConfig );
-    analyzer.setIndexWriter( *m_indexManager->indexWriter() );
-    analyzeFile( path, &analyzer );
+    Indexer indexer;
+    indexer.indexFile( QFileInfo( path ) );
 }
 
 
@@ -595,12 +530,10 @@ void Nepomuk::IndexScheduler::deleteEntries( const QStringList& entries )
 {
     // recurse into subdirs
     // TODO: use a less mem intensive method
-    std::vector<std::string> stdEntries;
     for ( int i = 0; i < entries.count(); ++i ) {
         deleteEntries( getChildren( entries[i] ).keys() );
-        stdEntries.push_back( QFile::encodeName( entries[i] ).data() );
+        m_indexer->clearIndexedData( KUrl( entries[i] ) );
     }
-    m_indexManager->indexWriter()->deleteEntries( stdEntries );
 }
 
 
@@ -682,7 +615,7 @@ void Nepomuk::IndexScheduler::removeOldAndUnwantedEntries()
     // We query all files that should not be in the store
     // This for example excludes all filex:/ URLs.
     //
-    QString query = QString::fromLatin1( "select distinct ?g ?url where { "
+    QString query = QString::fromLatin1( "select distinct ?g where { "
                                          "?r %1 ?url . "
                                          "?g <http://www.strigi.org/fields#indexGraphFor> ?r . "
                                          "FILTER(REGEX(STR(?url),'^file:/')) . "
@@ -690,27 +623,20 @@ void Nepomuk::IndexScheduler::removeOldAndUnwantedEntries()
                     .arg( Soprano::Node::resourceToN3( Nepomuk::Vocabulary::NIE::url() ),
                           folderFilter );
     kDebug() << query;
-
-    Soprano::QueryResultIterator it = ResourceManager::instance()->mainModel()->executeQuery( query, Soprano::Query::QueryLanguageSparql );
-    while ( it.next() ) {
-
-        // wait for resume or stop (or simply continue)
-        if ( !waitForContinue() ) {
-            break;
-        }
-
-        const Soprano::Node& g = it[0];
-        kDebug() << "REMOVING" << it["url"].uri();
-        ResourceManager::instance()->mainModel()->removeContext( g );
-    }
+    if ( !removeAllGraphsFromQuery( query ) )
+        return;
 
 
     //
     // Build filter query for all exclude filters
     //
     QStringList fileFilters;
-    foreach( const QRegExp& re, Nepomuk::StrigiServiceConfig::self()->excludeFilterRegExps() ) {
-        fileFilters << QString::fromLatin1( "REGEX(STR(?fn),\"^%1$\")" ).arg( re.pattern().replace( '\\',"\\\\" ) );
+    foreach( const QString& filter, Nepomuk::StrigiServiceConfig::self()->excludeFilters() ) {
+        QString filterRxStr = QRegExp::escape( filter );
+        filterRxStr.replace( "\\*", QLatin1String( ".*" ) );
+        filterRxStr.replace( "\\?", QLatin1String( "." ) );
+        filterRxStr.replace( '\\',"\\\\" );
+        fileFilters << QString::fromLatin1( "REGEX(STR(?fn),\"^%1$\")" ).arg( filterRxStr );
     }
     QString includeExcludeFilters = constructExcludeIncludeFoldersFilter();
 
@@ -721,8 +647,8 @@ void Nepomuk::IndexScheduler::removeOldAndUnwantedEntries()
         filters = QString::fromLatin1("FILTER(%1) .").arg( fileFilters.join(" || ") );
     else if( !includeExcludeFilters.isEmpty() )
         filters = QString::fromLatin1("FILTER(%1) .").arg( includeExcludeFilters );
-    
-    query = QString::fromLatin1( "select distinct ?g ?url where { "
+
+    query = QString::fromLatin1( "select distinct ?g where { "
                                  "?r %1 ?url . "
                                  "?r %2 ?fn . "
                                  "?g <http://www.strigi.org/fields#indexGraphFor> ?r . "
@@ -732,18 +658,8 @@ void Nepomuk::IndexScheduler::removeOldAndUnwantedEntries()
                   Soprano::Node::resourceToN3( Nepomuk::Vocabulary::NFO::fileName() ),
                   filters );
     kDebug() << query;
-    it = ResourceManager::instance()->mainModel()->executeQuery( query, Soprano::Query::QueryLanguageSparql );
-    while ( it.next() ) {
-
-        // wait for resume or stop (or simply continue)
-        if ( !waitForContinue() ) {
-            break;
-        }
-
-        const Soprano::Node& g = it[0];
-        kDebug() << "REMOVING" << it["url"].uri();
-        ResourceManager::instance()->mainModel()->removeContext( g );
-    }
+    if ( !removeAllGraphsFromQuery( query ) )
+        return;
 
 
     //
@@ -758,16 +674,63 @@ void Nepomuk::IndexScheduler::removeOldAndUnwantedEntries()
                                  "{ graph ?g { ?r2 %1 ?u2 . } } "
                                  "}" )
             .arg( Soprano::Node::resourceToN3( Soprano::Vocabulary::Xesam::url() ) );
-    it = ResourceManager::instance()->mainModel()->executeQuery( query, Soprano::Query::QueryLanguageSparql );
-    while ( it.next() ) {
+    kDebug() << query;
+    if ( !removeAllGraphsFromQuery( query ) )
+        return;
 
-        // wait for resume or stop (or simply continue)
-        if ( !waitForContinue() ) {
-            break;
+
+    //
+    // Remove data which is useless but still around from before. This could happen due to some buggy version of
+    // the indexer or the filewatch service or even some application messing up the data.
+    // We look for indexed files that do not have a nie:url defined and thus, will never be catched by any of the
+    // other queries. In addition we check for an isPartOf relation since strigi produces EmbeddedFileDataObjects
+    // for video and audio streams.
+    //
+    Query::Query q(
+        Strigi::Ontology::indexGraphFor() == ( Soprano::Vocabulary::RDF::type() == Query::ResourceTerm( Nepomuk::Vocabulary::NFO::FileDataObject() ) &&
+                                               !( Nepomuk::Vocabulary::NIE::url() == Query::Term() ) &&
+                                               !( Nepomuk::Vocabulary::NIE::isPartOf() == Query::Term() ) )
+        );
+    q.setQueryFlags(Query::Query::NoResultRestrictions);
+    query = q.toSparqlQuery();
+    kDebug() << query;
+    removeAllGraphsFromQuery( query );
+}
+
+
+
+/**
+ * Runs the query using a limit until all graphs have been deleted. This is not done
+ * in one big loop to avoid the problems with messed up iterators when one of the iterated
+ * item is deleted.
+ */
+bool Nepomuk::IndexScheduler::removeAllGraphsFromQuery( const QString& query )
+{
+    while ( 1 ) {
+        // get the next batch of graphs
+        QList<Soprano::Node> graphs
+            = ResourceManager::instance()->mainModel()->executeQuery( query + QLatin1String( " LIMIT 200" ),
+                                                                      Soprano::Query::QueryLanguageSparql ).iterateBindings( 0 ).allNodes();
+
+        // remove all graphs in the batch
+        Q_FOREACH( const Soprano::Node& graph, graphs ) {
+
+            // wait for resume or stop (or simply continue)
+            if ( !waitForContinue() ) {
+                return false;
+            }
+
+            ResourceManager::instance()->mainModel()->removeContext( graph );
         }
 
-        ResourceManager::instance()->mainModel()->removeContext( it[0] );
+        // we are done when the last graphs are queried
+        if ( graphs.count() < 200 ) {
+            return true;
+        }
     }
+
+    // make gcc shut up
+    return true;
 }
 
 
